#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# AS-250 version checker for Linux
# (c) 2018 Century Systems Co., Ltd.

import argparse
from logging import basicConfig, getLogger, DEBUG, INFO, WARNING, ERROR, CRITICAL
import os
import sys

from lib import exe
from lib import targethost
from lib.status import HostStatus

version='1.0'

def loginit(logLevelStr):
    global logger
    logLevelDict = {
        'DEBUG': DEBUG,
        'INFO': INFO,
        'WARNING': WARNING,
        'ERROR': ERROR,
        'CRITICAL': CRITICAL,
    }
    logLevel=logLevelDict.get(logLevelStr, WARNING)
    basicConfig(level=logLevel)
    logger = getLogger(__name__)

def parse_args():
    global version
    parser = argparse.ArgumentParser(
        description='Version checker for AS-250. (version {0})'.format(version))
    # 引数やオプションを解釈する部分
    parser.add_argument('--loglevel', metavar='LEVEL',
        choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
        help='log level (DEBUG|INFO|WARNING|ERROR|CRITICAL) (default: WARNING)',
        default='WARNING')
    parser.add_argument('-c', '--concurrent', type=int, metavar='NUM',
        help='Concurrent Execution Limit (default=20, max=100)', default=20)
    parser.add_argument('-r', '--resultfile', metavar='RESULTFILE',
        help='output of version list (default: resultfile.txt)',
        default='resultfile.txt')
    parser.add_argument('-l', '--listfile', action='store_true',
        help='ARG is file name of URL list (default: ARG is URL)')
    parser.add_argument('ARG',
        help='http(s)://[<password>@]<IPAddress>[:<port>] or filename')
    
    args = parser.parse_args()
    
    # concurrentがレンジ内かチェック
    # choicesを使うとエラーメッセージが冗長になるので使わない
    if args.concurrent < 1 or args.concurrent > 100:
        print("concurrent(-c / --concurrent) must in 1-100.")
        sys.exit(os.EX_USAGE)
    
    loginit(args.loglevel)
    # 解釈した引数を返す
    return args

def getListFromFile(filename):
    try:
        return targethost.TargetHost.getTargetHosts(filename)
    except Exception as ex:
        print("error in reading list file.")
        sys.exit(os.EX_USAGE)

def makeTargetHosts(args):
    concurrent = args.concurrent
    if args.listfile == True:
        (targetHosts, errorLines) = getListFromFile(args.ARG)
        if len(errorLines) > 0:
            for errorLine in errorLines:
                print("wrong target: {0}".format(errorLine))
            sys.exit(os.EX_USAGE)
    else:
        try:
            targetHost = targethost.TargetHost(args.ARG)
            targetHosts = [targetHost]
        except Exception:
            print("wrong target: {0}".format(args.ARG))
            sys.exit(os.EX_USAGE)
    
    return targetHosts

def openResultFile(args):
    if args.listfile == True:
        try:
            _file = open(args.resultfile, 'x')
            return _file
        except FileExistsError:
            print("result file exist.")
            sys.exit(os.EX_CANTCREAT)
    else:
        return sys.stdout

def doVersionCheck(args, targetHosts):
    global logger
    executor = exe.VersionCheck(args, logger=logger)
    return executor.execute(targetHosts)

def writeResultFile(resultFile, targetHosts):
    for resultHost in targetHosts:
        if resultHost.hostStatus == HostStatus.FINISHED:
            resultStr = "{0}: {1}\n".format(
                resultHost.getHost(), resultHost.getVersion())
        else:
            resultStr = "{0}: {1}\n".format(
                resultHost.getHost(), resultHost.errorstr)
        resultFile.write(resultStr)
    resultFile.close()

def makeReturnValue(targetHosts):
    for resultHost in targetHosts:
        if resultHost.hostStatus != HostStatus.FINISHED:
            return os.EX_TEMPFAIL
    else:
        return os.EX_OK

def main():
    args = parse_args()
    targetHosts = makeTargetHosts(args)
    resultFile = openResultFile(args)
    doVersionCheck(args, targetHosts)
    writeResultFile(resultFile, targetHosts)
    sys.exit(makeReturnValue(targetHosts))

if __name__ == "__main__":
    main()
