# wsse/client/requests/auth.py
# coding=utf-8
# pywsse
# Authors: Rushy Panchal, Naphat Sanguansin, Adam Libresco, Jérémie Lumbroso
# Date: September 1st, 2016
# Description: Authentication handler for requests library.

try:
    import requests
except ImportError:
    raise ImportError('requests is required to use the requests plugin')

import datetime
from wsse import settings, utils

class WSSEAuth2(requests.auth.AuthBase):
    '''
    Attaches HTTP Digest Authentication to the given Request object.

    Usage:
        requests.get(url, auth=WSSEAuth(username, password))

    The auth object isreusable, and can be defined separately if several
    requests will be made with the same username & password pair.

    :param username: username to include in request
    :type username: str

    :param password: password to authenticate with
    :type password: str
	'''
    def __init__(self, username, password):
        self.token_builder = TokenBuilder2(username, password, 'SHA1')

    def __call__(self, r):
        '''Attach the appropriate request header to attempt WSSE authentication.'''
        token = self.token_builder.make_token()
        r.headers[settings.REQUEST_HEADER] = "UsernameToken " + token
        return r

class TokenBuilder2(object):
    '''
    A `TokenBuilder` generates tokens for the given username and password.

    :param username: username for token authentication
    :type username: str

    :param password: password for token authentication
    :type password: str
    '''
    def __init__(self, username, password, algorithm):
        self.username = username
        self.password = password
        self.algorithm = algorithm

    def make_token(self, nonce = None, timestamp = None):
        '''
        Make a new WSSE token with the current username and password.
        
        :param nonce: nonce to use in token
        :type nonce: str
        
        :param timestamp: timestamp the nonce was generated at
        :type timestamp: datetime.datetime
        
        :param algorithm: algorithm for digest
        :type algorithm: str

        :return: WSSE token
        :rtype: str
        '''
        if not nonce:
            nonce = utils._random_string()
        if not timestamp:
            timestamp = datetime.datetime.utcnow()

        return utils.make_token(
            self.username, self.password, nonce, timestamp, None, self.algorithm)
