import time
import hashlib
import json
import re

import requests
from requests.exceptions import RequestException, HTTPError, ConnectTimeout, ConnectionError
from lib.auth import WSSEAuth2

from lib import status

class DecodeError(Exception):
    """APIからの返答のデコード時に発生する例外"""
    pass

class VersionCheck:
    """バージョンチェックを行うクラス"""
    
    endpoint_product = '/service/product'
    username = 'admin'
    
    def __init__(self, targetHost, *, logger=None):
        self._targetHost = targetHost
        self._logger = logger
    
    def docheck(self):
        """AS-250にWSSE認証をし、APIエンドポイントからバージョン番号を取得する"""
        errorstr = None
        try:
            self._targetHost.hostStatus = status.HostStatus.FAILED_DEAD
            auth = self._makeauth()
            response = self._getinfo(auth)
            response.raise_for_status()
            version = self._getversion(response)
            self._targetHost.setVersion(version)
            self._targetHost.hostStatus = status.HostStatus.FINISHED
        except (ConnectTimeout, ConnectionError):
            # 接続失敗
            errorstr = "Connection error."
        except HTTPError:
            # HTTPステータスがエラー
            self._targetHost.hostStatus = status.HostStatus.FAILED_ALIVE
            if response.status_code == 401:
                errorstr = "Unauthorized. Wrong password."
            else:
                errorstr = "HTTP error. Status = {0}.".format(
                    response.status_code)
        except RequestException as ex:
            # その他のHTTP通信エラー
            self._targetHost.hostStatus = status.HostStatus.FAILED_ALIVE
            errorstr = "RequestException {0}".format(str(ex))
        except DecodeError as ex:
            # APIから取得した情報を処理する際のデコードに失敗
            self._targetHost.hostStatus = status.HostStatus.FAILED_ALIVE
            errorstr = "Decoding error from {0}".format(self.endpoint_product)
        except Exception as ex:
            # その他のエラー
            errorstr = "Exception {0}".format(str(ex))
        finally:
            if errorstr is not None:
                self._logger.warning("{0}: {1}".format(
                    self._targetHost.getHost(), errorstr))
                self._targetHost.errorstr = errorstr

    def _makeauth(self):
        # AS-250では、パスワードを一度SHA1変換したものをWSSEの元として使う。
        s = hashlib.sha1()
        p = self._targetHost.getPassword()
        s.update(p.encode('ascii'))
        password = s.hexdigest()
        
        return WSSEAuth2(self.username, password)

    def _getinfo(self, auth):
        # APIから情報を取得
        url_product = self._targetHost.getURLBase() + self.endpoint_product
        
        self._logger.debug("GET {0}".format(url_product))
        
        # verify=falseにすると警告が出る。この警告を消すにはいくつか方法があるが、
        # urllib3.disable_warnings(InsecureRequestWarning) の方法は、
        # urllib3がrequestsの中にあるか外にあるか(requestsのバージョンによる)で
        # 対処法が異なるので使用しない。
        # 代わりにユーザーに環境変数を設定してもらう。README.txt参照。
        response = requests.get(url_product, auth = auth, verify = False)
        text = re.sub(r'\r|\n', '', response.text)
        self._logger.debug("response: {0}".format(text))
        return response

    def _getversion(self, response):
        try:
            product = json.loads(response.text)
            productname = product['product']
            major = product['ver_major']
            minor = product['ver_minor']
            revision = product['ver_revision']
            build = product['ver_build']
        
            version = "{0} v{1}.{2}.{3}.{4}".format(productname, major, minor, revision, build)
            return version
        except (json.JSONDecodeError, KeyError) as ex:
            raise DecodeError(ex)
